<?php if(!defined('PmWiki')) exit();

 /**
 *  Copyright 2011 Carlos A. Bonamigo <cabsec.pmwiki@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **/

/// HTMLCOMPRESS - Replaces action browse and compresses the html output of pmwiki

/// VARIABLES

$RecipeInfo['htmlcompress']['version'] = '20110416';

$HandleActions['browse'] = 'HandleBrowseHtmlCompress';
$HandleAuth['browse'] = 'read';

$HandleActions['compress'] = 'HandleHtmlCompress';
$HandleAuth['compress'] = 'read';

$EditFunctions[] = 'HtmlCompressCacheUpdate';

SDV($HtmlCompressCache, true );
SDV($HtmlCompressSuffix, ".html");
SDV($HtmlCompressFolder, "./wikiz.d/" );
SDV($HtmlCompressCachePerm, true);
SDV($HtmlCompressTabReturnNewline, array("\t", "\r", "\n"));
SDV($HtmlCompressSpacePat , '/\s{1,10}/');
SDV($HtmlCompressMoreSpace , array(
  "> $KeepToken" => ">$KeepToken",
  "$KeepToken <" => "$KeepToken<",
  "$KeepToken\n" => "$KeepToken",
  "\n$KeepToken" => "$KeepToken",
  "> <style" => "><style",
  "> <table" => "><table",
  "> <meta" => "><meta",
  "> <link" => "><link",
  "<br />" => "<br/>",
  "<hr />" => "<hr/>",
  "> </p>" => "></p>",
  "<p> " => "<p>",
  "> <div" => "><div",
  "> <td" => "><td",
  "> <h1" => "><h1",
  "> <h2" => "><h2",
  "> <h3" => "><h3",
  "> <h4" => "><h4",
  "> <h5" => "><h5",
  "> <h6" => "><h6",
  "--> <" => "--><",
  "<!-- " => "<!--",
  "'> </" => "'></",
  "'> <" => "'><",
  "' />" => "'/>",
  "/> <" => "/><",
  " </" => "</"
));

/// MARKUP

Markup('nohtmlcache',
       'directives',
       '/\\(:nohtmlcache:\\)/ei',
       "PZZ(\$GLOBALS['HtmlCompressCachePerm'] = false )"
);

/// FUNCTIONS

function HtmlCompressCacheUpdate($pagename, &$page, &$new) {
  global $IsPagePosted,$HtmlCompressCache,
      $HtmlCompressCachePerm,$HtmlCompressFolder,
      $HtmlCompressSuffix,$PageStartFmt,$PageEndFmt;

  if(!$IsPagePosted) return;
  $New = MarkupToHTML($pagename,$new['text']);
  if($HtmlCompressCache && $HtmlCompressCachePerm){
    SDV($HtmlCompressFmt,array(&$PageStartFmt, &$New, &$PageEndFmt));
    ob_start();
    HtmlCompressPrintFmt($pagename,$HtmlCompressFmt);
    $NewPage = ob_get_contents();
    ob_end_clean();
    $CompressedHtml = HtmlCompress($NewPage);
    $CacheFile = $HtmlCompressFolder.
          $pagename.$HtmlCompressSuffix;
    if(!file_exists($HtmlCompressFolder))
       mkdirp($HtmlCompressFolder);
    file_put_contents($CacheFile,$CompressedHtml);
  }
}

function HtmlCompressPrintFmt($pagename,$fmt) {
  global $HTTPHeaders,$FmtV;
  if(is_array($fmt)) 
    { foreach($fmt as $f) PrintFmt($pagename,$f); return; }
  if($fmt == 'headers:') {
    continue;
  }
  $x = FmtPageName($fmt,$pagename);
  if (strncmp($fmt, 'function:', 9) == 0 &&
      preg_match('/^function:(\S+)\s*(.*)$/s', $x, $match) &&
      function_exists($match[1]))
    { $match[1]($pagename,$match[2]); return; }
  if (strncmp($fmt, 'file:', 5) == 0 && preg_match("/^file:(.+)/s",$x,$match)) {
    $filelist = preg_split('/[\\s]+/',$match[1],-1,PREG_SPLIT_NO_EMPTY);
    foreach($filelist as $f) {
      if (file_exists($f)) { include($f); return; }
    }
    return;
  }
  if (substr($x, 0, 7) == 'markup:')
    { print MarkupToHTML($pagename, substr($x, 7)); return; }
  if (substr($x, 0, 5) == 'wiki:')
    { PrintWikiPage2($pagename, substr($x, 5), 'read'); return; }
  if (substr($x, 0, 5) == 'page:')
    { PrintWikiPage2($pagename, substr($x, 5), ''); return; }
  echo $x;
}


function HtmlCompress($h){
global $HtmlCompressCommentPat,$HtmlCompressTabReturnNewline,
  $HtmlCompressSpacePat,$HtmlCompressMoreSpace,
  $KeepToken;

  // remove one line comments
  $h = preg_replace("/\\<!\\-\\-[\\/A-Za-z]{3,30}\\-\\-\\>/",'', $h);

  // preserve pre and other html tags
  $t = "(textarea|pre|code)";
  $h = preg_replace(
      "/(\\<".$t."[^\>]*\\>.*\\<\\/".$t."\\>)/sUe",
      "Keep(PSS('$1'),'H')", $h);

  // compress in page javascript and preserve
  $h = preg_replace(
      "/(\\<script[^\>]*\\>)(.*)\\<\\/script\\>/sUe",
      "HtmlCompressJs(PSS('$1'),PSS('$2'))", $h);

  // compress in page css and preserve
  $h = preg_replace(
      "/(\\<style[^\>]*\\>)(.*)\\<\\/style\\>/sUe",
      "HtmlCompressCss(PSS('$1'),PSS('$2'))", $h);

  // turn newlines in to spaces inside p tags
  // otherwise words get joined togheter
  $h = preg_replace(
      "/(\\<p[^\>]*\\>.*\\<\\/p\\>)/sUe",
      "str_replace(chr('10'),' ',PSS('$1'))",$h);

  // remove tabs, returns and newlines
  $h = str_replace($HtmlCompressTabReturnNewline,'', $h);

  // replace spaces with just one space
  $h = preg_replace($HtmlCompressSpacePat,' ', $h);

  // more space
  $h = str_replace(array_keys($HtmlCompressMoreSpace),
         array_values($HtmlCompressMoreSpace), $h);

  //restore Keeps
  $h = preg_replace("/$KeepToken(\\d.*?)$KeepToken/e",
          '$GLOBALS[\'KPV\'][\'$1\']',$h);

return $h;
}

function HtmlCompressCss($t,$c){
if(function_exists('CssCompress'))
   $c = CssCompress($c);
$f = $t.$c."</style>";
$f = Keep($f,'C');
return $f;
}

function HtmlCompressJs($t,$j){
if(function_exists('JsCompress'))
   $j = JsCompress($j);
$f = $t.$j."</script>";
$f = Keep($f,'J');
return $f;
}

/// HANDLERS 

function HandleHtmlCompress($pagename,$auth='read'){
global $HtmlCompressFolder,$HtmlCompressSuffix;

  $CacheFileName = $HtmlCompressFolder.
          $pagename.$HtmlCompressSuffix;
  ob_start('HtmlCompress');
  HandleBrowse($pagename,$auth);
  $CompressedHtml = ob_get_contents();
  if(!file_exists($HtmlCompressFolder))
     mkdirp($HtmlCompressFolder);
  file_put_contents($CacheFile,$CompressedHtml);
  ob_end_flush();
}

function HandleBrowseHtmlCompress($pagename,$auth='read'){
global $HtmlCompressFolder,$HtmlCompressSuffix;

  $CacheFileName = $HtmlCompressFolder.
          $pagename.$HtmlCompressSuffix;

  if(file_exists($CacheFileName)){
    $page = RetrieveAuthPage($pagename, $auth, true, READPAGE_CURRENT);
    if(!$page) Abort("?cannot read $pagename");
    $CacheFileContent = file_get_contents($CacheFileName);
    echo $CacheFileContent; 
  }else{
    ob_start('HtmlCompress');
    HandleBrowse($pagename,$auth);
    ob_end_flush();
  }
}
